#include "keyboard.h"
#include "scan.h"
#include "save.h"
#include "sleep.h"

#define SCAN_TASK_CORE 0                  //CPU1
#define SCAN_TASK_PRIORITY 2              //优先级为1
#define SCAN_TASK_STACK 4096              //堆栈字数 堆栈位宽32 堆栈大小为4*2048字节
#define SCAN_TASK_DELAY 4                 //任务延时
TaskHandle_t SCAN_TASK_Handle = NULL;     //此任务由主任务打开和关闭
QueueHandle_t KEYBOARD_DATA_QUEUE = NULL; //键盘是否扫描

int RowCnt = 0;

void scan_task(void *pvParameters)
{
  uint8_t KEY_STATE[KEYBOARD_ROW][KEYBOARD_COLUMN] = {OFF};
  //Check columns //检查列
  for (;;)
  {
    if (get_scan_keyboard() == 1)
    {
      digitalWrite(Rows[RowCnt], HIGH);
      int ColCnt = 0;
      while (ColCnt <= (NumCols - 1))
      {
        if (digitalRead( Cols[ColCnt] ) == HIGH && KEY_STATE[RowCnt][ColCnt] == OFF)
        {
          KEY_STATE[RowCnt][ColCnt] = digitalRead(Cols[ColCnt]);
          KEY_STATE[RowCnt][ColCnt] = ON;
#ifdef DEBUG_K
          Serial.println("按键按下");
#endif
        }
        else if (digitalRead( Cols[ColCnt] ) == LOW && KEY_STATE[RowCnt][ColCnt] == ON)
        {
          KEY_STATE[RowCnt][ColCnt] = digitalRead(Cols[ColCnt]);
          KEY_STATE[RowCnt][ColCnt] = OFF;
#ifdef DEBUG_K
          Serial.println("按键松开");
#endif
        }
        ColCnt++;
        KEY_STATE[ENTER_X][ENTER_Y] = digitalRead(KEY_ENTER_PIN);
        xQueueOverwrite(KEYBOARD_SATE_QUEUE, &KEY_STATE);
      }
      digitalWrite(Rows[RowCnt], LOW);
      RowCnt++;
      if (RowCnt >= (NumRows))
      {
        RowCnt = 0;
      }
    }
#ifdef SCAN_TASK_DELAY
    vTaskDelay(SCAN_TASK_DELAY);
#endif
  }
  SCAN_TASK_Handle = NULL;
  vTaskDelete(NULL);
}

/**
   name: scan_task_begin

   返回值：
    成功返回1 失败返回0

   功能：
    开启SCAN任务
 * */
uint8_t scan_task_begin()
{
  if (SCAN_TASK_Handle != NULL)
  {
    return 1;
  }
  else
  {
    if (init_scan_queue() == 0)
    {
      return 0;
    }
    //创建扫描任务
    BaseType_t xReturned;
    xReturned = xTaskCreatePinnedToCore(scan_task, "scan_task", SCAN_TASK_STACK, NULL, SCAN_TASK_PRIORITY, &SCAN_TASK_Handle, SCAN_TASK_CORE);
    if (xReturned == pdPASS)
    {
      return 1;
    }
  }
  return 0;
}

/**
   name: scan_task_end

   功能：
    关闭SCAN任务
 * */
void scan_task_end()
{
  if (SCAN_TASK_Handle != NULL)
  {
    vTaskDelete(SCAN_TASK_Handle);
    SCAN_TASK_Handle = NULL;

    Config_scan config_scan;
    while (xQueueReceive(KEYBOARD_DATA_QUEUE, &config_scan, 0) == pdTRUE)
    {
    }
    vQueueDelete(KEYBOARD_DATA_QUEUE);
    KEYBOARD_DATA_QUEUE = NULL;

    uint8_t key_state[KEYBOARD_ROW][KEYBOARD_COLUMN] = {};
    while (xQueueReceive(KEYBOARD_SATE_QUEUE, &key_state, 0) == pdTRUE)
    {
    }
    vQueueDelete(KEYBOARD_SATE_QUEUE);
    KEYBOARD_SATE_QUEUE = NULL;
  }
}

/**
   name: init_scan_queue

   功能：
    初始化队列
 * */
uint8_t init_scan_queue()
{
  if (KEYBOARD_DATA_QUEUE == NULL && KEYBOARD_SATE_QUEUE == NULL)
  {
    //初始化 KEYBOARD_DATA_QUEUE
    Config_scan config_scan;
    KEYBOARD_DATA_QUEUE = xQueueCreate(1, sizeof(config_scan));
    if (KEYBOARD_DATA_QUEUE != NULL)
    {
      xQueueOverwrite(KEYBOARD_DATA_QUEUE, &config_scan);
    }
    else
    {
      return 0;
    }

    //初始化 KEYBOARD_SATE_QUEUE
    uint8_t key_state[KEYBOARD_ROW][KEYBOARD_COLUMN] = {};
    KEYBOARD_SATE_QUEUE = xQueueCreate(1, sizeof(key_state));
    if (KEYBOARD_SATE_QUEUE != NULL)
    {
      xQueueOverwrite(KEYBOARD_SATE_QUEUE, &key_state);
    }
    else
    {
      //KEYBOARD_SATE_QUEUE 初始化失败则将 KEYBOARD_DATA_QUEUE 释放
      while (xQueueReceive(KEYBOARD_DATA_QUEUE, &config_scan, 0) == pdTRUE)
      {
      }
      vQueueDelete(KEYBOARD_DATA_QUEUE);
      KEYBOARD_DATA_QUEUE = NULL;
      return 0;
    }
  }
  return 1;
}

/**
   name: set_scan_keyboard

   传参：
   0/1

   返回值：
    返回设置模式 错误返回ERROR_CODE

   功能：
    设置是否扫描 0/1
 **/
uint8_t set_scan_keyboard(uint8_t val)
{
  if (KEYBOARD_DATA_QUEUE != NULL)
  {
    Config_scan config_scan;
    xQueuePeek(KEYBOARD_DATA_QUEUE, &config_scan, 5);
    config_scan.scan = val;
    xQueueOverwrite(KEYBOARD_DATA_QUEUE, &config_scan);
    return config_scan.scan;
  }
  return ERROR_CODE;
}

/**
   name: get_scan_keyboard

   返回值：
    扫描返回1 不扫描返回0 错误返回ERROR_CODE

   功能：
    SCAN 任务是否扫描
 **/
uint8_t get_scan_keyboard()
{
  if (KEYBOARD_DATA_QUEUE != NULL)
  {
    Config_scan config_scan;
    xQueuePeek(KEYBOARD_DATA_QUEUE, &config_scan, 5);
    return config_scan.scan;
  }
  return ERROR_CODE;
}

/**
   name: fn_set

   传参：
    uint8_t state_ 按键状态
    uint8_t i 按键所在行
    uint8_t j 按键所在列

   功能：
    设置键盘
 * **/
void fn_set(uint8_t state_, uint8_t i, uint8_t j)
{
  if (i == USB_BLE_MODE_X && j == USB_BLE_MODE_Y) //USB键
  {
    if (state_ == 0)
    {
      Keyboard_Config.K_mode = USB_MODE;
#ifdef DEBUG_K
      Serial.println("切换为 USB");
#endif
      save_config();
      Wire.beginTransmission(0x2D); // Address of the STG8G
      Wire.write(0x06); // Value to write
      Wire.endTransmission();
      delay(100);
      clean_led();       
      esp_sleep_enable_timer_wakeup(1);
      esp_deep_sleep_start();
    }
  }
  else if (i == Bluetooth_channel_1_X && j == Bluetooth_channel_1_Y) //切换蓝牙通道1
  {
    if (state_ == 0)
    {
#ifdef DEBUG_K
      Serial.println("切换蓝牙通道1");
#endif
      Keyboard_Config.Bluetooth_channel = 0;
      Keyboard_Config.K_mode = BLE_MODE;
      Wire.beginTransmission(0x2D); // Address of the STG8G
      Wire.write(0x03); // Value to write
      Wire.endTransmission();
      delay(100);
      changeID(0);
    }
  }
  else if (i == Bluetooth_channel_2_X && j == Bluetooth_channel_2_Y) //切换蓝牙通道2
  {
    if (state_ == 0)
    {
#ifdef DEBUG_K
      Serial.println("切换蓝牙通道2");
#endif
      Keyboard_Config.Bluetooth_channel = 1;
      Keyboard_Config.K_mode = BLE_MODE;
      Wire.beginTransmission(0x2D); // Address of the STG8G
      Wire.write(0x04); // Value to write
      Wire.endTransmission();
      delay(100);
      changeID(1);
    }
  }
  else if (i == Bluetooth_channel_3_X && j == Bluetooth_channel_3_Y) //切换蓝牙通道3
  {
    if (state_ == 0)
    {
#ifdef DEBUG_K
      Serial.println("切换蓝牙通道3");
#endif
      Keyboard_Config.Bluetooth_channel = 2;
      Keyboard_Config.K_mode = BLE_MODE;
      Wire.beginTransmission(0x2D); // Address of the STG8G
      Wire.write(0x05); // Value to write
      Wire.endTransmission();
      delay(100);
      changeID(2);
    }
  }
  else if (i == Fn_W_X && j == Fn_W_Y) //切换为Windows键位
  {
    if (state_ == 0)
    {
      Keyboard_Config.Switch_system = WINDOWS_SYSTEM;
#ifdef DEBUG_K
      Serial.println("切换为Windows");
#endif
      save_config();
      vTaskDelay(50);
      load_config();
    }
  }
  else if (i == Fn_A_X && j == Fn_A_Y) //切换为Mac Os键位
  {
    if (state_ == 0)
    {
      Keyboard_Config.Switch_system = MAC_OS_SYSTEM;
#ifdef DEBUG_K
      Serial.println("切换为Mac OS");
#endif
      save_config();
      vTaskDelay(50);
      load_config();
    }
  }
  else if (i == Fn_UP_X && j == Fn_UP_Y) //亮度+键位
  {
    if (state_ == 0)
    {
#ifdef DEBUG_K
      Serial.println("亮度+");
#endif
      load_config();
      if (Keyboard_Config.L_brgightness < LIGHT_BRGIGHTNESS_MAX)
      {
        Keyboard_Config.L_brgightness += 20;
      }
      else
      {
        Keyboard_Config.L_brgightness = LIGHT_BRGIGHTNESS_MAX;
      }
      set_brgightness(Keyboard_Config.L_brgightness);
      save_config();
      FastLED.setBrightness(Keyboard_Config.L_brgightness);
    }
  }
  else if (i == Fn_DOWN_X && j == Fn_DOWN_Y) //亮度-键位
  {
    if (state_ == 0)
    {
#ifdef DEBUG_K
      Serial.println("亮度-");
#endif
      load_config();
      if (Keyboard_Config.L_brgightness > 0 && Keyboard_Config.L_brgightness <= LIGHT_BRGIGHTNESS_MAX)
      {
        Keyboard_Config.L_brgightness -= 20;
      }
      else
      {
        Keyboard_Config.L_brgightness = 0;
      }
      set_brgightness(Keyboard_Config.L_brgightness);
      save_config();
      FastLED.setBrightness(Keyboard_Config.L_brgightness);
    }
  }
  else if (i == Fn_RIGHT_X && j == Fn_RIGHT_Y) //灯速+键位
  {
    if (state_ == 0)
    {
#ifdef DEBUG_K
      Serial.println("灯速+");
#endif
      load_config();
      if (Keyboard_Config.L_speed > 10 && Keyboard_Config.L_speed <= LIGHT_SPEED_MAX)
      {
        Keyboard_Config.L_speed -= 10;
      }
      else
      {
        Keyboard_Config.L_speed = 10;
      }
      set_speed(Keyboard_Config.L_speed);
      save_config();
    }
  }
  else if (i == Fn_LEFT_X && j == Fn_LEFT_Y) //灯速-键位
  {
    if (state_ == 0)
    {
#ifdef DEBUG_K
      Serial.println("灯速-");
#endif
      load_config();
      if (Keyboard_Config.L_speed < LIGHT_SPEED_MAX)
      {
        Keyboard_Config.L_speed += 10;
      }
      else
      {
        Keyboard_Config.L_brgightness = LIGHT_SPEED_MAX;
      }

      set_speed(Keyboard_Config.L_speed);
      save_config();
    }
  }
  else if (i == Fn_DEL_X && j == Fn_DEL_Y) //切换RGB颜色
  {
    if (state_ == 0)
    {
#ifdef DEBUG_K
      Serial.println("切换RGB颜色");
#endif
      load_config();
      uint8_t color_temp = 0;
      if (get_rgb(&color_temp) >= 0)
      {
        if (color_temp < LIGHT_CORLOR_MAX && color_temp >= 0)
        {
          color_temp = color_temp + 1;
        }
        else
        {
          color_temp = 0;
        }
        Keyboard_Config.L_color = color_temp;
        set_rgb(color_temp);
        save_config();
      }
    }
  }
  else if (i == Fn_INS_X && j == Fn_INS_Y) //切换RGB灯效
  {
    if (state_ == 0)
    {
#ifdef DEBUG_K
      Serial.println("切换RGB灯效");
#endif
      load_config();
      if (Keyboard_Config.L_mode < LIGHT_MODE_NUM - 1)
      {
        Keyboard_Config.L_mode++;
        if (Keyboard_Config.L_mode == Horizontal_wave_mode)
        {
          Horizontal_wave_flag = 1;
        }
        Serial.print("模式编码：");
        Serial.println(Keyboard_Config.L_mode);
      }
      else
      {
        Keyboard_Config.L_mode = 0;
      }
      RGB_mode_change_flag = 1;
      set_light_mode(Keyboard_Config.L_mode);
      save_config();
    }
  }
  else if (i == Fn_6_X && j == Fn_6_Y) //切换为fps模式
  {
    if (state_ == 0)
    {
#ifdef DEBUG_K
      Serial.println("切换为fps模式");
#endif
      game_flag = 1;
      Keyboard_Config.L_mode = 13;
      set_light_mode(Keyboard_Config.L_mode);
      save_config();
    }
  }
}
